@@ -10,7 +10,8 @@ import os
 import traceback
 import threading
 from fireworks.core.firework import FWAction, FireWork
-from fireworks.core.fw_config import FWConfig, FWData
+from fireworks.fw_config import FWData, PING_TIME_SECS, REMOVE_USELESS_DIRS, PRINT_FW_JSON, PRINT_FW_YAML, STORE_PACKING_INFO
+from fireworks.utilities.dict_mods import apply_mod
 
 __author__ = 'Anubhav Jain'
 __copyright__ = 'Copyright 2013, The Materials Project'
@@ -29,10 +30,9 @@ def do_ping(launchpad, launch_id):
 def ping_launch(launchpad, launch_id, stop_event, master_thread):
     while not stop_event.is_set() and master_thread.isAlive():
         do_ping(launchpad, launch_id)
-        stop_event.wait(FWConfig().PING_TIME_SECS)
+        stop_event.wait(PING_TIME_SECS)
 
-
-def start_ping_launch(launch_id, lp):
+def start_ping_launch(launchpad, launch_id):
     fd = FWData()
     if fd.MULTIPROCESSING:
         if not launch_id:
@@ -43,11 +43,11 @@ def start_ping_launch(launch_id, lp):
     else:
         ping_stop = threading.Event()
         ping_thread = threading.Thread(target=ping_launch,
-                                       args=(lp, launch_id, ping_stop, threading.currentThread()))
+                                       args=(launchpad, launch_id, ping_stop, threading.currentThread()))
         ping_thread.start()
         return ping_stop
 
-def stop_ping_launch(ping_stop):
+def stop_backgrounds(ping_stop, btask_stops):
     fd = FWData()
     if fd.MULTIPROCESSING:
         m = fd.DATASERVER
@@ -55,6 +55,29 @@ def stop_ping_launch(ping_stop):
     else:
         ping_stop.set()
 
+    for b in btask_stops:
+        b.set()
+
+
+def background_task(btask, spec, stop_event, master_thread):
+    num_launched = 0
+    while not stop_event.is_set() and master_thread.isAlive():
+        for task in btask.tasks:
+            task.run_task(spec)
+        if btask.sleep_time > 0:
+            stop_event.wait(btask.sleep_time)
+
+        num_launched += 1
+        if num_launched == btask.num_launches:
+            break
+
+
+def start_background_task(btask, spec):
+    ping_stop = threading.Event()
+    ping_thread = threading.Thread(target=background_task, args=(btask, spec, ping_stop, threading.currentThread()))
+    ping_thread.start()
+    return ping_stop
+
 
 class Rocket():
     """
@@ -101,7 +124,7 @@ class Rocket():
             launch_id = None  # we don't need this in offline mode...
 
         if not m_fw:
-            print "No FireWorks are ready to run and match query! {}".format(self.fworker.query)
+            print("No FireWorks are ready to run and match query! {}".format(self.fworker.query))
             return False
 
         if '_launch_dir' in m_fw.spec:
@@ -112,25 +135,33 @@ class Rocket():
             if lp:
                 lp._change_launch_dir(launch_id, launch_dir)
 
-            if not os.listdir(prev_dir) and FWConfig().REMOVE_USELESS_DIRS:
+            if not os.listdir(prev_dir) and REMOVE_USELESS_DIRS:
                 try:
                     os.rmdir(prev_dir)
                 except:
                     pass
 
         # write FW.json and/or FW.yaml to the directory
-        if FWConfig().PRINT_FW_JSON:
+        if PRINT_FW_JSON:
             m_fw.to_file('FW.json', indent=4)
-        if FWConfig().PRINT_FW_YAML:
+        if PRINT_FW_YAML:
             m_fw.to_file('FW.yaml')
 
-        # set up heartbeat (pinging the server that we're still alive)
         try:
-            ping_stop = start_ping_launch(launch_id, lp)
+            my_spec = dict(m_fw.spec)  # make a copy of spec, don't override original
+
+            # set up heartbeat (pinging the server that we're still alive)
+            ping_stop = start_ping_launch(lp, launch_id)
+
+            # start background tasks
+            btask_stops = []
+            if '_background_tasks' in my_spec:
+                for bt in my_spec['_background_tasks']:
+                    btask_stops.append(start_background_task(bt, m_fw.spec))
 
             # execute the FireTasks!
             for my_task in m_fw.tasks:
-                m_action = my_task.run_task(m_fw.spec)
+                m_action = my_task.run_task(my_spec)
 
                 # read in a FWAction from a file, in case the task is not Python and cannot return it explicitly
                 if os.path.exists('FWAction.json'):
@@ -141,24 +172,39 @@ class Rocket():
                 if not m_action:
                     m_action = FWAction()
 
-                # update the global stored data with the data to store from this particular Task
+                # update the global stored data with the data to store and update from this particular Task
                 all_stored_data.update(m_action.stored_data)
                 all_update_spec.update(m_action.update_spec)
                 all_mod_spec.extend(m_action.mod_spec)
 
+                # update spec for next task as well
+                my_spec.update(m_action.update_spec)
+                for mod in m_action.mod_spec:
+                    apply_mod(mod, my_spec)
+
                 if m_action.skip_remaining_tasks:
                     break
 
             # add job packing info if this is needed
-            if FWData().MULTIPROCESSING and FWConfig().STORE_PACKING_INFO:
+            if FWData().MULTIPROCESSING and STORE_PACKING_INFO:
                 all_stored_data['multiprocess_name'] = multiprocessing.current_process().name
 
             # perform finishing operation
-            stop_ping_launch(ping_stop)
+            stop_backgrounds(ping_stop, btask_stops)
+            for b in btask_stops:
+                b.set()
             do_ping(lp, launch_id)  # one last ping, esp if there is a monitor
+            # last background monitors
+            if '_background_tasks' in my_spec:
+                for bt in my_spec['_background_tasks']:
+                    if bt.run_on_finish:
+                        for task in bt.tasks:
+                            task.run_task(m_fw.spec)
+
             m_action.stored_data = all_stored_data
             m_action.mod_spec = all_mod_spec
             m_action.update_spec = all_update_spec
+
             if lp:
                 lp.complete_launch(launch_id, m_action, 'COMPLETED')
             else:
@@ -174,9 +220,13 @@ class Rocket():
             return True
 
         except:
-            stop_ping_launch(ping_stop)
+            stop_backgrounds(ping_stop, btask_stops)
             traceback.print_exc()
-            m_action = FWAction(stored_data={'_message': 'runtime error during task', '_task': my_task.to_dict(),
+            try:
+                m_action = FWAction(stored_data={'_message': 'runtime error during task', '_task': my_task.to_dict(),
+                                             '_exception': traceback.format_exc()}, exit=True)
+            except:
+                m_action = FWAction(stored_data={'_message': 'runtime error during task', '_task': None,
                                              '_exception': traceback.format_exc()}, exit=True)
             if lp:
                 lp.complete_launch(launch_id, m_action, 'FIZZLED')
