@@ -26,14 +26,18 @@ Some advantages:
 A dict created using FWSerializer's to_dict() method should be readable by Pymatgen's PMGDecoder,
 when the serialize_fw() decorator is used.
 """
-
-import yaml
+import traceback
 import pkgutil
 import inspect
 import json  # note that ujson is faster, but at this time does not support "default" in dumps()
 import importlib
 import datetime
-from fireworks.core.fw_config import FWConfig
+import abc
+
+import yaml
+import six
+
+from fireworks.fw_config import FW_NAME_UPDATES, YAML_STYLE, USER_PACKAGES
 
 __author__ = 'Anubhav Jain'
 __copyright__ = 'Copyright 2012, The Materials Project'
@@ -46,18 +50,18 @@ SAVED_FW_MODULES = {}
 DATETIME_HANDLER = lambda obj: obj.isoformat() if isinstance(obj, datetime.datetime) else None
 
 
-def _recursive_dict(obj):
+def recursive_dict(obj):
     if obj is None:
         return None
 
     if hasattr(obj, 'to_dict'):
-        return _recursive_dict(obj.to_dict())
+        return recursive_dict(obj.to_dict())
 
     if isinstance(obj, dict):
-        return {k: _recursive_dict(v) for k, v in obj.items()}
+        return {k: recursive_dict(v) for k, v in obj.items()}
 
     if isinstance(obj, list):
-        return [_recursive_dict(v) for v in obj]
+        return [recursive_dict(v) for v in obj]
 
     if isinstance(obj, int) or isinstance(obj, float):
         return obj
@@ -65,7 +69,7 @@ def _recursive_dict(obj):
     if isinstance(obj, datetime.datetime):
         return obj.isoformat()
 
-    if isinstance(obj, unicode) and obj != obj.encode('ascii', 'ignore'):
+    if isinstance(obj, six.text_type) and obj != obj.encode('ascii', 'ignore'):
         return obj
 
     return str(obj)
@@ -84,7 +88,7 @@ def _recursive_load(obj):
     if isinstance(obj, list):
         return [_recursive_load(v) for v in obj]
 
-    if isinstance(obj, basestring):
+    if isinstance(obj, six.string_types):
         try:
             # convert String to datetime if really datetime
             return datetime.datetime.strptime(obj, "%Y-%m-%dT%H:%M:%S.%f")
@@ -104,7 +108,7 @@ def recursive_serialize(func):
 
     def _decorator(self, *args, **kwargs):
         m_dict = func(self, *args, **kwargs)
-        m_dict = _recursive_dict(m_dict)
+        m_dict = recursive_dict(m_dict)
         return m_dict
 
     return _decorator
@@ -139,12 +143,14 @@ def serialize_fw(func):
     return _decorator
 
 
-class FWSerializable():
+six.add_metaclass(abc.ABCMeta)
+class FWSerializable(object):
     """
-    To create a serializable object within FireWorks, you should subclass this class and implement
-    the to_dict() and from_dict() methods.
+    To create a serializable object within FireWorks, you should subclass this
+    class and implement the to_dict() and from_dict() methods.
 
-    If you want the load_object() implicit de-serialization to work, you must also:
+    If you want the load_object() implicit de-serialization to work, you must
+    also:
         - Use the @serialize_fw decorator on to_dict()
         - Override the _fw_name parameter with a unique key.
 
@@ -163,6 +169,7 @@ class FWSerializable():
         except AttributeError:
             return get_default_serialization(self.__class__)
 
+    @abc.abstractmethod
     def to_dict(self):
         raise NotImplementedError('FWSerializable object did not implement to_dict()!')
 
@@ -170,6 +177,7 @@ class FWSerializable():
         return self.to_dict()
 
     @classmethod
+    @abc.abstractmethod
     def from_dict(cls, m_dict):
         raise NotImplementedError('FWSerializable object did not implement from_dict()!')
 
@@ -182,7 +190,7 @@ class FWSerializable():
             return json.dumps(self.to_dict(), default=DATETIME_HANDLER, **kwargs)
         elif f_format == 'yaml':
             # start with the JSON format, and convert to YAML
-            return yaml.dump(self.to_dict(), default_flow_style=FWConfig().YAML_STYLE,
+            return yaml.dump(self.to_dict(), default_flow_style=YAML_STYLE,
                              allow_unicode=True)
         else:
             raise ValueError('Unsupported format {}'.format(f_format))
@@ -250,15 +258,13 @@ def load_object(obj_dict):
     """
 
     # override the name in the obj_dict if there's an entry in FW_NAME_UPDATES
-    fw_name = FWConfig().FW_NAME_UPDATES.get(obj_dict['_fw_name'], obj_dict['_fw_name'])
+    fw_name = FW_NAME_UPDATES.get(obj_dict['_fw_name'], obj_dict['_fw_name'])
     obj_dict['_fw_name'] = fw_name
 
     # check for explicit serialization, e.g. {{fireworks.tasks.MyTask}} - based on pymatgen method
     if fw_name.startswith('{{') and fw_name.endswith('}}'):
-        modclass = fw_name[2:-2].strip()
-        modname = modclass[0:modclass.rfind('.')]
-        classname = modclass[modclass.rfind('.')+1:]
-        mod = __import__(modname, globals(), locals(), [classname], -1)
+        modname, classname = fw_name.strip('{} ').rsplit(".", 1)
+        mod = __import__(modname, globals(), locals(), [classname], 0)
         if hasattr(mod, classname):
             cls_ = getattr(mod, classname)
             return cls_.from_dict(obj_dict)
@@ -274,32 +280,41 @@ def load_object(obj_dict):
     # this will be slow, but only needed the first time
 
     found_objects = [] # used to make sure we don't find multiple hits
-    for package in FWConfig().USER_PACKAGES:
+    for package in USER_PACKAGES:
         root_module = importlib.import_module(package)
-        for loader, module_name, is_pkg in pkgutil.walk_packages(root_module.__path__,
-                                                                 package + '.'):
-            m_module = loader.find_module(module_name).load_module(module_name)
-            m_object = _search_module_for_obj(m_module, obj_dict)
-            if m_object is not None:
-                found_objects.append((m_object, module_name))
+        for loader, mod_name, is_pkg in pkgutil.walk_packages(
+                root_module.__path__, package + '.'):
+            try:
+                m_module = loader.find_module(mod_name).load_module(mod_name)
+                m_object = _search_module_for_obj(m_module, obj_dict)
+                if m_object is not None:
+                    found_objects.append((m_object, mod_name))
+            except ImportError as ex:
+                import warnings
+                warnings.warn(
+                    "%s in %s cannot be loaded because of %s. Skipping.."
+                    % (m_object, mod_name, str(ex)))
+                traceback.print_exc(ex)
 
     if len(found_objects) == 1:
         SAVED_FW_MODULES[fw_name] = found_objects[0][1]
         return found_objects[0][0]
     elif len(found_objects) > 0:
         raise ValueError(
-            'load_object() found multiple objects with cls._fw_name {} -- {}'.format(fw_name,
-                                                                                     found_objects))
+            'load_object() found multiple objects with cls._fw_name {} -- {}'
+            .format(fw_name, found_objects))
 
     raise ValueError('load_object() could not find a class with cls._fw_name {}'.format(fw_name))
 
 
 def load_object_from_file(filename, f_format=None):
     """
-    implicitly load an object from a file. just a friendly wrapper to load_object()
+    Implicitly load an object from a file. just a friendly wrapper to
+    load_object()
 
     :param filename: the filename to load an object from
-    :param f_format: the serialization format (default is auto-detect based on filename extension)
+    :param f_format: the serialization format (default is auto-detect based on
+        filename extension)
     """
 
     m_dict = {}
@@ -326,7 +341,7 @@ def _search_module_for_obj(m_module, obj_dict):
     for name, obj in inspect.getmembers(m_module):
         # check if the member is a Class matching our description
         if inspect.isclass(obj) and obj.__module__ == m_module.__name__ and \
-                        getattr(obj, '_fw_name', get_default_serialization(obj)) == obj_name:
+                getattr(obj, '_fw_name', get_default_serialization(obj)) == obj_name:
             return obj.from_dict(obj_dict)
 
 
@@ -340,7 +355,7 @@ def _reconstitute_dates(obj_dict):
     if isinstance(obj_dict, list):
         return [_reconstitute_dates(v) for v in obj_dict]
 
-    if isinstance(obj_dict, basestring):
+    if isinstance(obj_dict, six.string_types):
         try:
             return datetime.datetime.strptime(obj_dict, "%Y-%m-%dT%H:%M:%S.%f")
         except:
@@ -352,5 +367,8 @@ def _reconstitute_dates(obj_dict):
 def get_default_serialization(cls):
     root_mod = cls.__module__.split('.')[0]
     if root_mod == '__main__':
-        raise ValueError("Cannot get default serialization; try instantiating your object from a different module from which it is defined rather than defining your object in the __main__ (running) module.")
-    return root_mod + '::' + cls.__name__  # e.g. fireworks.ABC
\ No newline at end of file
+        raise ValueError("Cannot get default serialization; try "
+                         "instantiating your object from a different module "
+                         "from which it is defined rather than defining your "
+                         "object in the __main__ (running) module.")
+    return root_mod + '::' + cls.__name__  # e.g. fireworks.ABC
