@@ -1,6 +1,4 @@
 #!/usr/bin/env python
-from bz2 import BZ2File
-from gzip import GzipFile
 
 import logging
 import datetime
@@ -12,7 +10,8 @@ import traceback
 import socket
 import multiprocessing
 
-from fireworks.core.fw_config import FWConfig, FWData
+from fireworks.fw_config import FWData, FW_BLOCK_FORMAT, DS_PASSWORD, \
+    FW_LOGGING_FORMAT
 
 
 __author__ = 'Anubhav Jain, Xiaohui Qu'
@@ -24,10 +23,11 @@ __date__ = 'Dec 12, 2012'
 
 PREVIOUS_STREAM_LOGGERS = []  # contains the name of loggers that have already been initialized
 PREVIOUS_FILE_LOGGERS = []  # contains the name of file loggers that have already been initialized
-DEFAULT_FORMATTER = logging.Formatter(FWConfig().FW_LOGGING_FORMAT)
+DEFAULT_FORMATTER = logging.Formatter(FW_LOGGING_FORMAT)
 
 
-def get_fw_logger(name, l_dir=None, file_levels=('DEBUG', 'ERROR'), stream_level='DEBUG', formatter=DEFAULT_FORMATTER,
+def get_fw_logger(name, l_dir=None, file_levels=('DEBUG', 'ERROR'),
+                  stream_level='DEBUG', formatter=DEFAULT_FORMATTER,
                   clear_logs=False):
     """
     Convenience method to return a logger.
@@ -125,15 +125,24 @@ def create_datestamp_dir(root_dir, l_logger, prefix='block_'):
     :param prefix: the prefix for the new dir, default="block_"
     """
 
-    time_now = datetime.datetime.utcnow().strftime(FWConfig().FW_BLOCK_FORMAT)
-    block_path = prefix + time_now
-    full_path = os.path.join(root_dir, block_path)
+    def get_path():
+        time_now = datetime.datetime.utcnow().strftime(FW_BLOCK_FORMAT)
+        block_path = prefix + time_now
+        return os.path.join(root_dir, block_path)
+
+    full_path = get_path()
+    while os.path.exists(full_path):
+        import time
+        import random
+        time.sleep(random.random()/3+0.1)
+        full_path = get_path()
+
     os.mkdir(full_path)
+
     l_logger.info('Created new dir {}'.format(full_path))
     return full_path
 
 
-
 def get_my_ip():
     try:
         return socket.gethostbyname(socket.gethostname())
@@ -166,7 +175,7 @@ class DataServer(BaseManager):
         """
         DataServer.register('LaunchPad', callable=lambda: launchpad)
         DataServer.register('Running_IDs', callable=lambda: {}, proxytype=DictProxy)
-        m = DataServer(address=('127.0.0.1', 0), authkey=FWConfig().DS_PASSWORD)  # random port
+        m = DataServer(address=('127.0.0.1', 0), authkey=DS_PASSWORD)  # random port
         m.start()
         return m
 
@@ -186,80 +195,6 @@ class NestedClassGetter(object):
         return nested_class()
 
 
-def reverse_readline(m_file, blk_size=4096, max_mem=4000000):
-    """
-    Generator method to read a file line-by-line, but backwards. This allows
-    one to efficiently get data at the end of a file.
-
-    Based on code by Peter Astrand <astrand@cendio.se>, using modifications by
-    Raymond Hettinger and Kevin German.
-    http://code.activestate.com/recipes/439045-read-a-text-file-backwards
-    -yet-another-implementat/
-
-    Reads file forwards and reverses in memory for files smaller than the
-    max_mem parameter, or for gzip files where reverse seeks are not supported.
-
-    Files larger than max_mem are dynamically read backwards.
-
-    Args:
-        m_file:
-            File stream to read (backwards)
-        blk_size:
-            The buffer size. Defaults to 4096.
-        max_mem:
-            The maximum amount of memory to involve in this operation. This is
-            used to determine when to reverse a file in-memory versus seeking
-            portions of a file. For bz2 files, this sets the maximum block
-            size.
-
-    Returns:
-        Generator that returns lines from the file. Similar behavior to the
-        file.readline() method, except the lines are returned from the back
-        of the file.
-    """
-
-    file_size = os.path.getsize(m_file.name)
-
-    # If the file size is within our desired RAM use, just reverse it in memory
-    # GZip files must use this method because there is no way to negative seek
-    if file_size < max_mem or isinstance(m_file, GzipFile):
-        for line in reversed(m_file.readlines()):
-            yield line.rstrip()
-    else:
-        if isinstance(m_file, BZ2File):
-            # for bz2 files, seeks are expensive. It is therefore in our best
-            # interest to maximize the blk_size within limits of desired RAM
-            # use.
-            blk_size = min(max_mem, file_size)
-
-        buf = ""
-        m_file.seek(0, 2)
-        lastchar = m_file.read(1)
-        trailing_newline = (lastchar == "\n")
-
-        while 1:
-            newline_pos = buf.rfind("\n")
-            pos = m_file.tell()
-            if newline_pos != -1:
-                # Found a newline
-                line = buf[newline_pos + 1:]
-                buf = buf[:newline_pos]
-                if pos or newline_pos or trailing_newline:
-                    line += "\n"
-                yield line
-            elif pos:
-                # Need to fill buffer
-                toread = min(blk_size, pos)
-                m_file.seek(pos - toread, 0)
-                buf = m_file.read(toread) + buf
-                m_file.seek(pos - toread, 0)
-                if pos == toread:
-                    buf = "\n" + buf
-            else:
-                # Start-of-file
-                return
-
-
 def explicit_serialize(o):
-    o._fw_name = '{{'+o.__module__+'.'+o.__name__+'}}'
+    o._fw_name = '{{%s.%s}}' % (o.__module__, o.__name__)
     return o
\ No newline at end of file
